        <thead>
            <tr>
                <th>Profile</th>
                <th>Image</th>
                <th>Numcpus</th>
                <th>Memory</th>
                <th>Pool</th>
                <th>Disks</th>
                <th>Nets</th>
                <th>Cloudinit</th>
                <th>Nested</th>
                <th>Reservedns</th>
                <th>Reservehost</th>
                <th>Actions</th>
            </tr>
        </thead>
        <tbody>
            {% for profile in profiles %}
            <tr>
                <td>{{ profile[0] }}</td>
                <td>{{ profile[5] }}</td>
                <td>{{ profile[1] }}</td>
                <td>{{ profile[2] }}</td>
                <td>{{ profile[3] }}</td>
                <td>{{ profile[4] }}</td>
                <td>{{ profile[6] }}</td>
                <td>{{ profile[7] }}</td>
                <td>{{ profile[8] }}</td>
                <td>{{ profile[9] }}</td>
                <td>{{ profile[10] }}</td>
                <td>
                    {% if not readonly %}
                    <button type="button" class="btn btn-default" id="create" onclick="vmcreate('', '{{ profile[0] }}');"><span data-toggle="tooltip" title="Create Vm from Profile" class="glyphicon glyphicon-shopping-cart"></span></button><p>
                    {% endif %}
                </td>
            </tr>
            {% endfor %} 
        </tbody>
