{% include "head.html" %}

{% if result != 'success' %}
<div class="alert alert-danger" role="alert">
Plan {{ plan }} not created because of<p>
{{ reason|replace('\n', '<p>') }}<p>
{% if failedvms %}
{% for v in failedvms %}
{{ v['name'] }} {{ v['reason'] }}<p>
{% endfor %}
{% endif %}
</div>
{% elif not newvms %}
<div class="alert alert-info" role="alert">
Plan {{ plan }} left unchanged
</div>
{% else %}
<div class="alert alert-success" role="alert">
Plan {{ plan }} created
</div>
{% endif %}
<p><a href="/">Back to listing</a>
</body>
</html>
