{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createpool">
    <div class="form-group">
        <label for="inputName">Pool</label>
        <input required="true" class="form-control" id="pool" placeholder="Enter Pool Name">
    </div>
    <div class="form-group">
        <label for="inputName">Path</label>
        <input required="true" class="form-control" id="path" placeholder="Enter Pool Path">
    </div>
    <div class="form-group">
        <label for="inputName">Type</label>
        <select class="form-control" id="type">
        <option value="dir" selected >Directory</option>
        <option value="logical">LVM</option>
        </select>
    </div>
    <button type="submit" class="btn btn-primary" onclick="poolcreate();">Create</button>
</form>

{% endblock %}
