{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createkube">
    {% for parameter in parameters|sort %}
    <div class="form-group">
        <label for="inputName">{{ parameter }}</label>
        <input required="true" class="form-control" name="{{ parameter }}" id="{{ parameter }}" value="{{ parameters[parameter] }}">
    </div>
    {% endfor %}
    <input type="hidden" id="type" name="type" value="{{ _type }}">
</form>
    <button type="submit" class="btn btn-primary" onclick="kubecreate();">Create</button>

{% endblock %}
