{
    "ignition": {
        "config": {
           "merge": [
              {
              "source": "{{ ignition_url }}"
              }
           ]
        },
    "version": "{{ ignition_version }}"
{% if hosts_content != None %}
    },
    "storage": {
        "files": [
            {
                "contents": {
                    "source": "data:text/plain;charset=utf-8;base64,{{ hosts_content }}",
                    "verification": {}
                },
                "filesystem": "root",
                "mode": 420,
                "overwrite": true,
                "path": "/etc/hosts"
            }
        ]
    }
{% else %}
    }
{% endif %}
}
