{% extends "layout.html" %}
{% block content %}

{% if not readonly %}
<a href='/kubeconfig/kubeconfig.{{ kube }}'>Get Kubeconfig</a><p>
{% endif %}
Version: {{ version }}<p>
<table class="table table-striped table-bordered" cellspacing="0" width="100%">
  <tbody>
  <tr>
    <th>Name</th>
    <th>Status</th>
    <th>Role</th>
    <th>Age</th>
    <th>Version</th>
    <th>IP</th>
  </tr>
  <tr>
{% for node in nodes %}
  <td>{{ node[0]}}</td>
  <td>{{ node[1]}}</td>
  <td>{{ node[2]}}</td>
  <td>{{ node[3]}}</td>
  <td>{{ node[4]}}</td>
  <td>{{ node[5]}}</td>
  </tr>
{% endfor %}
  </tbody>
</table>

{% endblock %}
