        <thead>
            <tr>
                <th>Name</th>
                <th>Type</th>
                <th>Enabled</th>
                <th>Default</th>
                <th>Actions</th>
            </tr>
        </thead>
        <tbody>
            {% for client in clients %}
                <tr>
                <td>{{ client[0] }}</td>
                <td>{{ client[1] }}</td>
                <td>{{ client[2] }}</td>
                <td>{{ client[3] }}</td>
                <td>
                    <div class="dropdown">
                        <button class="btn btn-default dropdown-toggle" type="button" id="actions" data-toggle="dropdown" aria-haspopup="true" aria-expanded="true"><font size="1">Actions</font>
                    <span class="caret"></span>
                    </button>
                    <ul class="dropdown-menu" aria-labelledby="dropdownMenu1">
                        {% if not readonly %}
                        <li onclick="hostenable('{{ client[0] }}');"><span class="glyphicon glyphicon-ok-circle green" data-toggle="tooltip" title="Enable"></span> Enable</li>
                        <li class="divider"></li>
                        <li onclick="hostdisable('{{ client[0] }}');"><span class="glyphicon glyphicon-remove-circle" data-toggle="tooltip" title="Disable"></span> Disable</li>
                        <li class="divider"></li>
                        <li onclick="hostswitch('{{ client[0] }}');"><span class="glyphicon glyphicon-heart-empty" data-toggle="tooltip" title="Switch"></span> Switch</li>
                        {% endif %}
                    </ul></div>
                </td>
            </tr>
            {% endfor %} 
        </tbody>
