apiVersion: v1
kind: Config
current-context: my-cluster
contexts: [{name: my-cluster, context: {cluster: cluster-1, user: user-1}}]
users: [{name: user-1, user: {exec: {apiVersion: client.authentication.k8s.io/v1beta1, args: [{{ client }}], command: gketoken, interactiveMode: Never}}}]
clusters:
- name: cluster-1
  cluster:
    server: https://{{ endpoint }}
    certificate-authority-data: {{ ca_cert }}
