{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createnetwork">
    <div class="form-group">
        <label for="inputName">Network</label>
        <input required="true" class="form-control" id="network" placeholder="Enter Network Name">
    </div>
    <div class="form-group">
        <label for="inputName">Cidr</label>
        <input required="true" class="form-control" id="cidr" placeholder="Enter Cidr">
    </div>
    <div class="checkbox">
        <label><input id="dhcp" type="checkbox" class="form-check-input" checked >Dhcp</label>
    </div>
    <div class="checkbox">
        <label><input id="isolated" type="checkbox" class="form-check-input" >Isolated</label>
    </div>
    <button type="submit" class="btn btn-primary" onclick="networkcreate();">Create</button>
</form>
{% endblock %}
