{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createcontainerform">
    <div class="form-group">
        <label for="inputName">Name</label>
        <input required="true" type="name" class="form-control" id="name" placeholder="Name">
    </div>
    <div class="form-group">
    <label for="profile">Select Profile:</label>
    <select class="form-control" id="profile">
      {% for profile in profiles %}
      <option>{{ profile[0] }}</option>
      {% endfor %}
    </select>
    </div>
</form>
    <button type="submit" id="createcontainerbutton" class="btn btn-primary" onclick="containercreate();">Create</button>

{% endblock %}
