<!DOCTYPE html>
<html class="layout-pf">
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="description" content="Libvirt/VirtualBox wrapper on steroids">
    <!--meta http-equiv="refresh" content="10" /-->
    <title>Kcli</title>
    <link rel="shortcut icon" href="/static/images/favicon.ico">

    {% block scripts %}
    <script src="/static/js/jquery.min.js"></script>
    <script src="/static/js/jquery.dataTables.min.js"></script>
    <script src="/static/js/dataTables.checkboxes.min.js"></script>
    <script src="/static/js/bootstrap.min.js"></script>
    <script src="/static/js/vmaction.js"></script>
    <script src="/static/js/containeraction.js"></script>
    <script src="/static/js/planaction.js"></script>
    <script src="/static/js/hostaction.js"></script>
    <script src="/static/js/networkaction.js"></script>
    <script src="/static/js/poolaction.js"></script>
    <script src="/static/js/snapshotaction.js"></script>
    <script src="/static/js/imageaction.js"></script>
    <script src="/static/js/repoaction.js"></script>
    <script src="/static/js/productaction.js"></script>
    <script src="/static/js/kubeaction.js"></script>
    <script src="/static/js/kcli.js"></script>
    <script src="/static/js/list.js"></script>
    <script src="/static/js/bootstrap-notify.js"></script>
    {% endblock %}

    <link href="https://fonts.googleapis.com/css2?family=Open+Sans:wght@300;400&family=Roboto:wght@100&family=Water+Brush&display=swap" rel="stylesheet">
    {% block stylesheets %}
    <link href="/static/css/jquery.dataTables.min.css" rel="stylesheet">
    <link href="/static/css/bootstrap.min.css" rel="stylesheet">
    <link href="/static/css/dataTables.checkboxes.css" rel="stylesheet" />
    <link href="/static/css/navbar.css" rel="stylesheet">
    <link href="/static/css/wheel.css" rel="stylesheet">
    <link href="/static/css/bootstrap-notify.css" rel="stylesheet">
    <link href="/static/css/kcli.css" rel="stylesheet">
    {% endblock %}
</head>
