<html>
<head>
<script src="https://code.jquery.com/jquery-3.5.1.min.js" integrity="sha256-9/aliU8dGd2tb6OSsuzixeV4y/faTqgFtohetphbbj0=" crossorigin="anonymous"></script>
<link rel="shortcut icon" href="/static/images/favicon.ico">
<script src="/static/js/exposeactions.js"></script>
<script src="/static/js/jquery.min.js"></script>
<script src="/static/js/jquery.dataTables.min.js"></script>
<script src="/static/js/dataTables.checkboxes.min.js"></script>
<script src="/static/js/list.js"></script>

<link href="https://fonts.googleapis.com/css2?family=Open+Sans:wght@300;400&family=Roboto:wght@100&family=Water+Brush&display=swap" rel="stylesheet">
<link href="/static/css/jquery.dataTables.min.css" rel="stylesheet">
<link href="/static/css/bootstrap.min.css" rel="stylesheet">
<link href="/static/css/dataTables.checkboxes.css" rel="stylesheet" />
<link href="/static/css/wheel.css" rel="stylesheet">
<link href="/static/css/bootstrap-notify.css" rel="stylesheet">
<link href="/static/css/kcli.css" rel="stylesheet">
</head>
<body>
