{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createvmform">
    <div class="form-group">
        <label for="inputName">Name</label>
        <input type="name" class="form-control" id="name" placeholder="Name">
    </div>
    <div class="form-group">
    <label for="profile">Select Image:</label>
    <select class="form-control" id="profile">
      {% for profile in profiles %}
      <option>{{ profile }}</option>
      {% endfor %}
    </select>
    </div>
    {% for parameter in parameters %}
    <div class="form-group">
        <label for="inputName">{{ parameter }}</label>
        <input required="true" class="form-control" name="{{ parameter }}" id="{{ parameter }}" value="{{ parameters[parameter] }}">
    </div>
    {% endfor %}
</form>
    <button type="submit" id="createvmbutton" class="btn btn-primary" onclick="vmcreate();">Create</button>

{% endblock %}
