        <thead>
            <tr>
                <th>Repo</th>
                <th>Group</th>
                <th>Name</th>
                <th>Description</th>
                <th>NumVms</th>
                <th>Actions</th>
            </tr>
        </thead>
        <tbody>
            {% for product in products %}
            <tr>
                <td>{{ product[0] }}</td>
                <td>{{ product[1] }}</td>
                <td>{{ product[2] }}</td>
                <td>{{ product[3] }}</td>
                <td>{{ product[4] }}</td>
                <td>
                    {% if not readonly %}
                    <a href="/productcreate/{{ product[2]) }}"><span data-toggle="tooltip" title="Create Product" class="glyphicon glyphicon-shopping-cart"></span></a><p>
                    {% endif %}
                </td>
            </tr>
            {% endfor %} 
        </tbody>
