{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createrepo">
    <div class="form-group">
        <label for="inputName">Repo</label>
        <input required="true" class="form-control" id="repo" placeholder="Enter Repo Name">
    </div>
    <div class="form-group">
        <label for="inputName">Url</label>
        <input required="true" class="form-control" id="URL" placeholder="Enter Repo Url">
    </div>
    <!--button type="submit" class="btn btn-primary" onclick="repocreate();">Create</button-->
</form>
    <button type="submit" class="btn btn-primary" onclick="repocreate();">Create</button>

{% endblock %}
