{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createproduct">
    <div class="form-group">
        <label for="inputName">Product</label>
        <input readonly="readonly" class="form-control" id="product" value="{{ product }}">
    </div>
    <div class="form-group">
        <label for="inputName">Description</label>
        <input readonly="readonly" class="form-control" id="description" value="{{ description }}">
    </div>
    <div class="form-group">
        <label for="inputName">Comments</label>
        <input readonly="readonly" class="form-control" id="info" value="{{ info }}">
    </div>
    <div class="form-group">
        <label for="inputName">Plan</label>
        <input class="form-control" id="plan" placeholder="Enter Plan Name or leave empty for auto generation">
    </div>
    {% for parameter in parameters %}
    <div class="form-group">
        <label for="inputName">{{ parameter }}</label>
        <input required="true" class="form-control" name="{{ parameter }}" id="{{ parameter }}" value="{{ parameters[parameter] }}">
    </div>
    {% endfor %}
    <button type="submit" class="btn btn-primary" onclick="productcreate();">Create</button>
</form>

{% endblock %}
