DHCPKEYWORDS = ['T1',
                'T2',
                'all-subnets-local',
                'arp-timeout',
                'boot-file-size',
                'bootfile-name',
                'bootfile-param',
                'bootfile-url',
                'classless-static-route',
                'client-arch',
                'client-interface-id',
                'client-machine-id',
                'default-ttl',
                'dns-server',
                'domain-name',
                'domain-search',
                'ethernet-encap',
                'extension-path',
                'information-refresh-time',
                'ip-forward-enable',
                'irc-server',
                'log-server',
                'lpr-server',
                'max-datagram-reassembly',
                'mobile-ip-home',
                'netbios-dd',
                'netbios-nodetype',
                'netbios-ns',
                'netbios-scope',
                'netmask',
                'nis+-domain',
                'nis+-server',
                'nis-domain',
                'nis-server',
                'nntp-server',
                'non-local-source-routing',
                'ntp-server',
                'policy-filter',
                'pop3-server',
                'root-path',
                'router',
                'router-discovery',
                'router-solicitation',
                'server-ip-address',
                'sip-server',
                'sip-server-domain',
                'smtp-server',
                'sntp-server',
                'static-route',
                'swap-server',
                'tcp-keepalive',
                'tcp-ttl',
                'tftp-server',
                'time-offset',
                'trailer-encapsulation',
                'user-class',
                'vendor-class',
                'vendor-id-encap',
                'x-windows-dm',
                'x-windows-fs']
