        <thead>
            <tr>
                <th>Isos</th>
            </tr>
        </thead>
        <tbody>
            {% for iso in isos %}
            <tr>
                <td>{{ iso }}</td>
            </tr>
            {% endfor %} 
        </tbody>
