{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createimageform">
    <div class="form-group">
    <label for="profile">Select Image:</label>
    <select class="form-control" id="image" onchange="imageurl();">
      {% for image in images %}
      <option url="{{ images[image] }}">{{ image }}</option>
      {% endfor %}
    </select>
    </div>
    <div class="form-group">
    <label for="pool">Select Pool:</label>
    <select class="form-control" id="pool">
      {% for pool in pools %}
      <option>{{ pool }}</option>
      {% endfor %}
    </select>
</div>
    <div class="form-group">
    <label for="inputName" id="urllabel" hidden >Url</label>
    <input type="text" class="form-control" id="url" placeholder="Url"/>
    </div>
    <div class="form-group">
    <label for="inputName" id="cmd" >Cmd to launch on the image</label>
    <input type="text" class="form-control" id="cmd" placeholder="Cmd"/>
    </div>
</form>
    <button type="submit" class="btn btn-primary" onclick="imagecreate();">Create</button>

{% endblock %}
