{% extends "layout.html" %}
{% block content %}

<div id="wheel"></div>
<div class='notifications top-right'></div>
<form id="createplanform">
    <div class="form-group">
        <label for="inputName">Plan Name</label>
        <input type="name" class="form-control" id="name" placeholder="Enter Name or Leave empty to have one autogenerated">
    </div>
    <div class="form-group">
        <label for="inputName">Url</label>
        <input class="form-control" id="planurl" placeholder="Enter Url">
    </div>
</form>
    <button type="submit" class="btn btn-primary" onclick="plancreate();">Create</button>
<p><p>Don't know which plans to use? Start with <a href="https://github.com/karmab/kcli-plan-samples/tree/main">here!</a>

{% endblock %}
