[connection]
ipv6.dhcp-duid=ll
ipv6.dhcp-iaid=mac
{% if sno_disable_nics|default([]) %}
{% set sno_disable_nics = sno_disable_nics|join(";interface-name:") %}
[keyfile]
unmanaged-devices=interface-name:{{ sno_disable_nics }}
{% endif %}
