        <thead>
            <tr>
                <th>Network</th>
                <th>Type</th>
                <th>Cidr</th>
                <th>Dhcp</th>
                <th>Mode</th>
                <th>Actions</th>
            </tr>
        </thead>
        <tbody>
            {% for network in networks %}
            <tr>
                <td>{{ network }}</td>
                <td>{{ networks[network]['type'] }}</td>
                <td>{{ networks[network]['cidr'] }}</td>
                <td>{{ networks[network]['dhcp'] }}</td>
                <td>{{ networks[network]['mode'] }}</td>
                {% if not readonly %}
                <td onclick="networkdelete('{{ network }}');"><span class="glyphicon glyphicon-trash"></span> Delete</td>
                {% endif %}
            </tr>
            {% endfor %} 
        </tbody>
